%% switch_core
% Implements the redistribution of weights under sequential updates in the
% switch distribution.

classdef switch_core < core
    
    properties (SetAccess = private)
        N = 0  % number of processed outcomes
        Kst    % number of strategies that no longer switch
    end
    
    methods

        %% switch_core(K_stable, K_switching)
        % Constructs a switch core for the given numbers of PFSs
        function this = switch_core(K_switching, K_stable)
            p_stabilize = mu();
            w_st = p_stabilize     * arrayfun(@(l)lambda(l, K_stable, 0), 1:K_stable);
            w_sw = (1-p_stabilize) * arrayfun(@(k)kappa(k, K_switching, 0), 1:K_switching);
            this = this@core([w_st w_sw]);
            this.Kst = K_stable;
        end

        %% update(p)
        % Updates the weights for all prediction strategies based on the
        % probabilities they assigned to the observed outcome.
        function update(this, p)
            this.update@core(p); % do the multiplicative update
           
            % redistribute weight
            this.N = this.N+1;
            p_switch = tau(this.N);
            p_stabilize = mu();
            w_st = this.weights(1:this.Kst);
            w_sw = this.weights(this.Kst+1:end);
            kst = this.Kst;
            ksw = length(w_sw);

            % recalculate weights for switching PFSs
            pool = p_switch * sum(w_sw);
            w_from_pool = pool * (1-p_stabilize) * arrayfun(@(k)kappa(k, ksw, this.N), 1:ksw);
            w_sw = (1-p_switch) * w_sw + w_from_pool;

            % recalculate weights for stable PFSs
            w_from_pool = pool * p_stabilize * arrayfun(@(l)lambda(l, kst, this.N), 1:kst);
            w_st = w_st + w_from_pool;
            
            % update weights
            this.weights = [w_st w_sw];
        end
        
    end
        
end



%% specification of the prior

% tau(n) returns the conditional probability tau(T=n | T>=n)
function p = tau(n); p = 1/(n+1); end

% mu() returns the conditional probability mu(M=m | M >= m), which
% has to be constant because m is unknown by the algorithm.
function p = mu(); p = 0.5; end

% kappa(k, kmax, n) returns the probability kappa_n(K=k), domain 1,...,kmax
function p = kappa(k, kmax, n); p = 1/kmax; end

% lambda(l, lmax, n) returns the probability lambda_n(L=l), domain 1,...,lmax
function p = lambda(l, lmax, n); p = 1/lmax; end
